import { FuseBox, QuantumPlugin, EnvPlugin } from "fuse-box"
import { NgCompilerPlugin } from "./tools/plugins/ng.compiler.plugin"
import { argv } from 'yargs'
import { NgPolyfillPlugin } from "./tools/plugins/ng.polyfill.plugin"
import { NgProdPlugin } from "./tools/plugins/ng.prod.plugin"
import { NgOptimizerPlugin } from "./tools/plugins/ng.optimizer.plugin"
import { NgServerModulePlugin } from "./tools/plugins/ng.server.module.plugin"
import { NgBootstrapPlugin } from "./tools/plugins/ng.bootstrap.plugin"

export interface FusingAngularConfig {
  productionBuild?: boolean
  supportIE11?: boolean
  supportIE11Animations?: boolean
  enableAotCompilaton?: boolean
  enableAngularAnimations?: boolean
  enableAngularForms?: boolean
  enableServiceWorker?: boolean
  enableAngularBuildOptimizer?: boolean
  serverPort?: number

  angularServerModule?: {
    moduleName: string
    modulePath: string
  }
}

const DEFAULT_CONFIG: FusingAngularConfig = {
  enableAotCompilaton: argv.aot,
  productionBuild: argv.prod,
  supportIE11: true,
  supportIE11Animations: false,
  enableAngularAnimations: false,
  enableAngularForms: false,
  enableServiceWorker: false,
  serverPort: process.env.PORT && +process.env.PORT || 5000,
  angularServerModule: {
    moduleName: 'AppServerModule',
    modulePath: 'src/server/server.angular.module'
  }
}

export const fusingAngular = (opts?: FusingAngularConfig) => {
  const settings = {
    ...DEFAULT_CONFIG,
    ...opts
  }
  return {
    run: () => {
      const fuseBrowser = FuseBox.init({
        homeDir: "./",
        output: "./.dist/public/js/$name.js",
        target: 'browser@es5',
        cache: !settings.productionBuild,
        plugins: [
          NgBootstrapPlugin({ enabled: settings.enableAotCompilaton }),
          NgProdPlugin({ enabled: settings.productionBuild }),
          NgPolyfillPlugin(),
          NgCompilerPlugin({ enabled: settings.enableAotCompilaton }),
          NgOptimizerPlugin({ enabled: settings.enableAngularBuildOptimizer }),
          settings.productionBuild && QuantumPlugin({
            bakeApiIntoBundle: 'vendor',
            warnings: false,
            uglify: true,
            treeshake: true,
            ensureES5: true
          })
        ] as any
      })

      const mainAppEntry = 'app.browser.module.ts'

      const fuseServer = FuseBox.init({
        target: 'server@es5',
        homeDir: "./",
        output: "./.dist/$name.js",
        cache: !settings.productionBuild,
        plugins: [
          EnvPlugin({ serverPort: settings.serverPort }),
          NgProdPlugin({ enabled: settings.productionBuild, fileTest: 'server.angular.module' }),
          settings.angularServerModule && NgServerModulePlugin({
            module: settings.angularServerModule.moduleName,
            path: settings.angularServerModule.modulePath
          }),
          NgPolyfillPlugin({ isServer: true, fileTest: 'server.angular.module' })
        ] as any[]
      })

      const vendorBundle = fuseBrowser.bundle('vendor')
      const appBundle = fuseBrowser.bundle('app')
      const serverBundle = fuseServer.bundle("server")

      vendorBundle.instructions(` ~ src/browser/${mainAppEntry}`)
      appBundle.instructions(` !> [src/browser/${mainAppEntry}]`)
      serverBundle
        .instructions(" > [src/server/server.ts]")
        .completed(proc => proc.start())

      if (!settings.productionBuild) {
        appBundle.watch('src/**')
        serverBundle.watch("src/**")
        fuseServer.dev({
          root: './.dist',
          port: settings.serverPort,
          httpServer: false
        })
      }

      fuseBrowser.run()
      fuseServer.run()
    }
  }
}

const app = fusingAngular({
  productionBuild: true,
  enableAotCompilaton: true
})


app.run()